﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading;

/*
 *    Copyright John-Daniel Trask
 *    http://blog.bluecog.co.nz
 */

namespace ParrallelTest
{
  internal class Program
  {
    private static readonly int _iterations = 25;
    private static readonly int _maxSize = 100000;

    private static void Main(string[] args)
    {
      List<long> parallelResults = new List<long>();
      List<long> sequentialResults = new List<long>();

      Console.WriteLine("RUNNING SEQUENTIAL TESTS");

      for (int i = 0; i < _iterations; i++)
      {
        sequentialResults.Add(RunPrimeFind());
        Console.WriteLine("Test run {0}: {1}", i + 1, sequentialResults[i]);
      }

      Console.WriteLine("RUNNING PARALLEL TESTS");

      for (int i = 0; i < _iterations; i++)
      {
        parallelResults.Add(RunPrimeFindParallel());
        Console.WriteLine("Test run {0}: {1}", i + 1, parallelResults[i]);
      }

      Console.WriteLine("SEQUENTIAL AVERAGE: {0}", sequentialResults.Average());
      Console.WriteLine("PARALLEL AVERAGE:   {0}", parallelResults.Average());

      Console.ReadLine();
    }

    private static long RunPrimeFindParallel()
    {
      Stopwatch stopwatch = new Stopwatch();
      stopwatch.Start();

      Parallel.For(0, _maxSize, i =>
      {
        for (int n = 2; n < (i); n++)
        {
          if (i % n == 0)
          {
            return; // Not prime
          }
        }
      });

      stopwatch.Stop();

      return stopwatch.ElapsedMilliseconds;
    }

    private static long RunPrimeFind()
    {
      Stopwatch stopwatch = new Stopwatch();
      stopwatch.Start();

      for (int i = 0; i < _maxSize; i++)
      {
        for (int n = 2; n < (i); n++)
        {
          if (i % n == 0)
          {
            break; // Not prime
          }
        }
      }

      stopwatch.Stop();

      return stopwatch.ElapsedMilliseconds;
    }
  }
}